@@ -7,16 +7,12 @@
 #include "loader.h"
 
 #include <dirent.h>
+#include <limits.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
 #include <time.h>
-
-// Default configuration parameters
-#define CFG_ORDER_DEF     "alpha"
-#define CFG_LOOP_DEF      true
-#define CFG_RECURSIVE_DEF false
-#define CFG_ALL_DEF       true
+#include <unistd.h>
 
 /** Context of the image list (which is actually an array). */
 struct image_list {
@@ -40,15 +36,84 @@ static const char* order_names[] = {
     [order_random] = "random",
 };
 
+/**
+ * Get absolute path from relative source.
+ * @param source relative file path
+ * @return absolute path, the caller should free the buffer
+ */
+static char* absolute_path(const char* source)
+{
+    char* abs_path = NULL;
+    char path[PATH_MAX];
+    struct str_slice dirs[1024];
+    size_t len;
+
+    if (strcmp(source, LDRSRC_STDIN) == 0 ||
+        strncmp(source, LDRSRC_EXEC, LDRSRC_EXEC_LEN) == 0) {
+        return str_dup(source, NULL);
+    }
+
+    if (*source == '/') {
+        strncpy(path, source, sizeof(path) - 1);
+    } else {
+        // relative to the current dir
+        if (!getcwd(path, sizeof(path) - 1)) {
+            return str_dup(source, NULL);
+        }
+        len = strlen(path);
+        if (path[len] != '/') {
+            path[len++] = '/';
+        }
+        strncpy(path + len, source, sizeof(path) - len - 1);
+    }
+
+    // split by component
+    len = str_split(path, '/', dirs, ARRAY_SIZE(dirs));
+
+    // remove "/../" and "/./"
+    for (size_t i = 0; i < len; ++i) {
+        if (dirs[i].len == 1 && dirs[i].value[0] == '.') {
+            dirs[i].len = 0;
+        } else if (dirs[i].len == 2 && dirs[i].value[0] == '.' &&
+                   dirs[i].value[1] == '.') {
+            dirs[i].len = 0;
+            for (ssize_t j = (ssize_t)i - 1; j >= 0; --j) {
+                if (dirs[j].len != 0) {
+                    dirs[j].len = 0;
+                    break;
+                }
+            }
+        }
+    }
+
+    // collect to the absolute path
+    str_append("/", 1, &abs_path);
+    for (size_t i = 0; i < len; ++i) {
+        if (dirs[i].len) {
+            str_append(dirs[i].value, dirs[i].len, &abs_path);
+            if (i < len - 1) {
+                str_append("/", 1, &abs_path);
+            }
+        }
+    }
+
+    return abs_path;
+}
+
 /**
  * Add new entry to the list.
  * @param source image data source to add
  */
 static void add_entry(const char* source)
 {
+    char* abs_source = absolute_path(source);
+    if (!abs_source) {
+        return;
+    }
+
     // check for duplicates
     for (size_t i = 0; i < ctx.size; ++i) {
-        if (strcmp(ctx.sources[i], source) == 0) {
+        if (strcmp(ctx.sources[i], abs_source) == 0) {
             return;
         }
     }
@@ -65,7 +130,7 @@ static void add_entry(const char* source)
     }
 
     // add new entry
-    ctx.sources[ctx.size] = str_dup(source, NULL);
+    ctx.sources[ctx.size] = abs_source;
     if (ctx.sources[ctx.size]) {
         ++ctx.size;
     }
@@ -222,32 +287,17 @@ static void shuffle_list(void)
  * Load config.
  * @param cfg config instance
  */
-static void load_config(struct config* cfg)
+static void load_config(const struct config* cfg)
 {
-    ssize_t index;
-    const char* order;
-
-    // list order
-    ctx.order = order_alpha;
-    order =
-        config_get_string(cfg, IMGLIST_SECTION, IMGLIST_ORDER, CFG_ORDER_DEF);
-    index = str_index(order_names, order, 0);
-    if (index >= 0) {
-        ctx.order = index;
-    } else {
-        config_error_val(IMGLIST_SECTION, IMGLIST_ORDER);
-    }
-
-    // list modes
-    ctx.loop =
-        config_get_bool(cfg, IMGLIST_SECTION, IMGLIST_LOOP, CFG_LOOP_DEF);
-    ctx.recursive = config_get_bool(cfg, IMGLIST_SECTION, IMGLIST_RECURSIVE,
-                                    CFG_RECURSIVE_DEF);
-    ctx.all_files =
-        config_get_bool(cfg, IMGLIST_SECTION, IMGLIST_ALL, CFG_ALL_DEF);
+    ctx.order = config_get_oneof(cfg, CFG_LIST, CFG_LIST_ORDER, order_names,
+                                 ARRAY_SIZE(order_names));
+    ctx.loop = config_get_bool(cfg, CFG_LIST, CFG_LIST_LOOP);
+    ctx.recursive = config_get_bool(cfg, CFG_LIST, CFG_LIST_RECURSIVE);
+    ctx.all_files = config_get_bool(cfg, CFG_LIST, CFG_LIST_ALL);
 }
 
-size_t image_list_init(struct config* cfg, const char** sources, size_t num)
+size_t image_list_init(const struct config* cfg, const char** sources,
+                       size_t num)
 {
     struct stat file_stat;
 
