@@ -9,6 +9,7 @@
 #include "fetcher.h"
 #include "imagelist.h"
 #include "info.h"
+#include "pixmap_scale.h"
 #include "ui.h"
 
 #include <stdio.h>
@@ -23,18 +24,6 @@
 #define GRID_COLOR1 0xff333333
 #define GRID_COLOR2 0xff4c4c4c
 
-// Default configuration parameters
-#define CFG_WINDOW_DEF         ARGB(0, 0, 0, 0)
-#define CFG_TRANSPARENCY_DEF   GRID_NAME
-#define CFG_SCALE_DEF          "optimal"
-#define CFG_POSITION_DEF       "center"
-#define CFG_FIXED_DEF          true
-#define CFG_ANTIALIASING_DEF   false
-#define CFG_SLIDESHOW_DEF      false
-#define CFG_SLIDESHOW_TIME_DEF 3
-#define CFG_HISTORY_DEF        1
-#define CFG_PRELOAD_DEF        1
-
 // Scale thresholds
 #define MIN_SCALE 10    // pixels
 #define MAX_SCALE 100.0 // factor
@@ -88,14 +77,15 @@ static const char* position_names[] = {
 
 /** Viewer context. */
 struct viewer {
-    ssize_t img_x, img_y; ///< Top left corner of the image
-    size_t frame;         ///< Index of the current frame
-    argb_t image_bkg;     ///< Image background mode/color
-    argb_t window_bkg;    ///< Window background mode/color
-    bool antialiasing;    ///< Anti-aliasing mode on/off
-    bool fixed;           ///< Fix image position
+    ssize_t img_x, img_y;        ///< Top left corner of the image
+    size_t frame;                ///< Index of the current frame
+    argb_t image_bkg;            ///< Image background mode/color
+    argb_t window_bkg;           ///< Window background mode/color
+    enum pixmap_aa_mode aa_mode; ///< Anti-aliasing mode
+    bool fixed;                  ///< Fix image position
 
     enum fixed_scale scale_init; ///< Initial scale
+    bool keep_zoom;              ///< Keep absolute zoom across images
     enum position position;      ///< Initial position
     float scale;                 ///< Current scale factor of the image
 
@@ -162,6 +152,56 @@ static void fixup_position(bool force)
     }
 }
 
+/** Set image position. */
+static void set_position(void)
+{
+    const struct image* img = fetcher_current();
+    const struct pixmap* pm = &img->frames[ctx.frame].pm;
+    const size_t wnd_width = ui_get_width();
+    const size_t wnd_height = ui_get_height();
+
+    switch (ctx.position) {
+        case position_top:
+            ctx.img_y = 0;
+            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
+            break;
+        case position_center:
+            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
+            break;
+        case position_bottom:
+            ctx.img_y = wnd_height - ctx.scale * pm->height;
+            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
+            break;
+        case position_left:
+            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+            ctx.img_x = 0;
+            break;
+        case position_right:
+            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
+            ctx.img_x = wnd_width - ctx.scale * pm->width;
+            break;
+        case position_top_left:
+            ctx.img_y = 0;
+            ctx.img_x = 0;
+            break;
+        case position_top_right:
+            ctx.img_y = 0;
+            ctx.img_x = wnd_width - ctx.scale * pm->width;
+            break;
+        case position_bottom_left:
+            ctx.img_y = wnd_height - ctx.scale * pm->height;
+            ctx.img_x = 0;
+            break;
+        case position_bottom_right:
+            ctx.img_y = wnd_height - ctx.scale * pm->height;
+            ctx.img_x = wnd_width - ctx.scale * pm->width;
+            break;
+    }
+
+    fixup_position(true);
+}
+
 /**
  * Move image (viewport).
  * @param horizontal axis along which to move (false for vertical)
@@ -256,46 +296,7 @@ static void scale_image(enum fixed_scale sc)
             break;
     }
 
-    switch (ctx.position) {
-        case position_top:
-            ctx.img_y = 0;
-            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
-            break;
-        case position_center:
-            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
-            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
-            break;
-        case position_bottom:
-            ctx.img_y = wnd_height - ctx.scale * pm->height;
-            ctx.img_x = wnd_width / 2 - (ctx.scale * pm->width) / 2;
-            break;
-        case position_left:
-            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
-            ctx.img_x = 0;
-            break;
-        case position_right:
-            ctx.img_y = wnd_height / 2 - (ctx.scale * pm->height) / 2;
-            ctx.img_x = wnd_width - ctx.scale * pm->width;
-            break;
-        case position_top_left:
-            ctx.img_y = 0;
-            ctx.img_x = 0;
-            break;
-        case position_top_right:
-            ctx.img_y = 0;
-            ctx.img_x = wnd_width - ctx.scale * pm->width;
-            break;
-        case position_bottom_left:
-            ctx.img_y = wnd_height - ctx.scale * pm->height;
-            ctx.img_x = 0;
-            break;
-        case position_bottom_right:
-            ctx.img_y = wnd_height - ctx.scale * pm->height;
-            ctx.img_x = wnd_width - ctx.scale * pm->width;
-            break;
-    }
-
-    fixup_position(true);
+    set_position();
     info_update(info_scale, "%.0f%%", ctx.scale * 100);
 }
 
@@ -429,10 +430,10 @@ static void reset_state(void)
     const size_t total_img = image_list_size();
 
     ctx.frame = 0;
-    ctx.img_x = 0;
-    ctx.img_y = 0;
-    ctx.scale = 0;
-    scale_image(ctx.scale_init);
+    if (!ctx.keep_zoom || ctx.scale == 0) {
+        scale_image(ctx.scale_init);
+    }
+    set_position();
 
     ui_set_title(img->name);
     animation_ctl(true);
@@ -578,7 +579,7 @@ static void draw_image(struct pixmap* wnd)
     if (img->alpha) {
         if (ctx.image_bkg == GRID_BKGID) {
             pixmap_grid(wnd, ctx.img_x, ctx.img_y, width, height,
-                        ui_get_scale() * GRID_STEP, GRID_COLOR1, GRID_COLOR2);
+                        GRID_STEP, GRID_COLOR1, GRID_COLOR2);
         } else {
             pixmap_fill(wnd, ctx.img_x, ctx.img_y, width, height,
                         ctx.image_bkg);
@@ -589,13 +590,7 @@ static void draw_image(struct pixmap* wnd)
     if (ctx.scale == 1.0) {
         pixmap_copy(img_pm, wnd, ctx.img_x, ctx.img_y, img->alpha);
     } else {
-        enum pixmap_scale scaler;
-        if (ctx.antialiasing) {
-            scaler = (ctx.scale > 1.0) ? pixmap_bicubic : pixmap_average;
-        } else {
-            scaler = pixmap_nearest;
-        }
-        pixmap_scale(scaler, img_pm, wnd, ctx.img_x, ctx.img_y, ctx.scale,
+        pixmap_scale(ctx.aa_mode, img_pm, wnd, ctx.img_x, ctx.img_y, ctx.scale,
                      img->alpha);
     }
 }
@@ -714,9 +709,11 @@ static void apply_action(const struct action* action)
             app_redraw();
             break;
         case action_antialiasing:
-            ctx.antialiasing = !ctx.antialiasing;
-            info_update(info_status, "Anti-aliasing %s",
-                        ctx.antialiasing ? "on" : "off");
+            if (++ctx.aa_mode >= ARRAY_SIZE(pixmap_aa_names)) {
+                ctx.aa_mode = 0;
+            }
+            info_update(info_status, "Anti-aliasing: %s",
+                        pixmap_aa_names[ctx.aa_mode]);
             app_redraw();
             break;
         case action_reload:
@@ -748,54 +745,36 @@ static void on_drag(int dx, int dy)
     }
 }
 
-void viewer_init(struct config* cfg, struct image* image)
+void viewer_init(const struct config* cfg, struct image* image)
 {
     size_t history;
     size_t preload;
     const char* value;
-    ssize_t index;
 
-    ctx.fixed =
-        config_get_bool(cfg, VIEWER_SECTION, VIEWER_FIXED, CFG_FIXED_DEF);
-    ctx.antialiasing = config_get_bool(cfg, VIEWER_SECTION, VIEWER_ANTIALIASING,
-                                       CFG_ANTIALIASING_DEF);
-    ctx.window_bkg =
-        config_get_color(cfg, VIEWER_SECTION, VIEWER_WINDOW, CFG_WINDOW_DEF);
+    ctx.fixed = config_get_bool(cfg, CFG_VIEWER, CFG_VIEW_FIXED);
+    ctx.aa_mode =
+        config_get_oneof(cfg, CFG_VIEWER, CFG_VIEW_AA, pixmap_aa_names,
+                         ARRAY_SIZE(pixmap_aa_names));
+    ctx.window_bkg = config_get_color(cfg, CFG_VIEWER, CFG_VIEW_WINDOW);
 
     // background for transparent images
-    value = config_get_string(cfg, VIEWER_SECTION, VIEWER_TRANSPARENCY,
-                              CFG_TRANSPARENCY_DEF);
+    value = config_get(cfg, CFG_VIEWER, CFG_VIEW_TRANSP);
     if (strcmp(value, GRID_NAME) == 0) {
         ctx.image_bkg = GRID_BKGID;
     } else {
-        ctx.image_bkg = config_get_color(cfg, VIEWER_SECTION,
-                                         VIEWER_TRANSPARENCY, GRID_BKGID);
+        ctx.image_bkg = config_get_color(cfg, CFG_VIEWER, CFG_VIEW_TRANSP);
     }
 
-    // initial scale
-    value = config_get_string(cfg, VIEWER_SECTION, VIEWER_SCALE, CFG_SCALE_DEF);
-    index = str_index(scale_names, value, 0);
-    if (index >= 0) {
-        ctx.scale_init = index;
-    } else {
-        ctx.scale_init = scale_fit_optimal;
-        config_error_val(VIEWER_SECTION, value);
-    }
-
-    value = config_get_string(cfg, VIEWER_SECTION, VIEWER_POSITION,
-                              CFG_POSITION_DEF);
-    index = str_index(position_names, value, 0);
-    if (index >= 0) {
-        ctx.position = index;
-    } else {
-        ctx.position = position_center;
-    }
+    // initial scale and position
+    ctx.scale_init = config_get_oneof(cfg, CFG_VIEWER, CFG_VIEW_SCALE,
+                                      scale_names, ARRAY_SIZE(scale_names));
+    ctx.keep_zoom = config_get_bool(cfg, CFG_VIEWER, CFG_VIEW_KEEP_ZM);
+    ctx.position = config_get_oneof(cfg, CFG_VIEWER, CFG_VIEW_POSITION,
+                                    position_names, ARRAY_SIZE(position_names));
 
     // cache and preloads
-    history = config_get_num(cfg, VIEWER_SECTION, VIEWER_HISTORY, 0, 1024,
-                             CFG_HISTORY_DEF);
-    preload = config_get_num(cfg, VIEWER_SECTION, VIEWER_PRELOAD, 0, 1024,
-                             CFG_PRELOAD_DEF);
+    history = config_get_num(cfg, CFG_VIEWER, CFG_VIEW_HISTORY, 0, 1024);
+    preload = config_get_num(cfg, CFG_VIEWER, CFG_VIEW_PRELOAD, 0, 1024);
 
     // setup animation timer
     ctx.animation_enable = true;
@@ -804,13 +783,10 @@ void viewer_init(struct config* cfg, struct image* image)
     if (ctx.animation_fd != -1) {
         app_watch(ctx.animation_fd, on_animation_timer, NULL);
     }
-
     // setup slideshow timer
-    ctx.slideshow_enable = config_get_bool(cfg, VIEWER_SECTION,
-                                           VIEWER_SLIDESHOW, CFG_SLIDESHOW_DEF);
+    ctx.slideshow_enable = config_get_bool(cfg, CFG_VIEWER, CFG_VIEW_SSHOW);
     ctx.slideshow_time =
-        config_get_num(cfg, VIEWER_SECTION, VIEWER_SLIDESHOW_TIME, 1, 86400,
-                       CFG_SLIDESHOW_TIME_DEF);
+        config_get_num(cfg, CFG_VIEWER, CFG_VIEW_SSHOW_TM, 1, 86400);
     ctx.slideshow_fd =
         timerfd_create(CLOCK_MONOTONIC, TFD_CLOEXEC | TFD_NONBLOCK);
     if (ctx.slideshow_fd != -1) {
